# Copyright (c) 2008-2009 Code Lounge.
# See LICENSE for details

from zope.interface import Interface, Attribute

class ICommand(Interface):
    name = Attribute("""
    A C{str} that is the command's name.
    """)

    usage = Attribute("""
    A single-line C{str} that details usage/syntax information for the command.
    """)

    help = Attribute("""
    A C{str}, optionally multi-line, that goes into detail about this command's
    behaviour and any related information.
    """)

    def locateCommand(params):
        """
        Find the C{ICommand} handler responsible for a given command.

        The first parameter is popped from C{params} and is treated as the
        command handler name. If no handler is found L{UsageError} is raised.

        @param params: The parameters, including the command, for a command
        @type params: C{list}

        @return: The command handler for the command and the remaining parameters
        @rtype: C{(ICommand, list)}
        """

class IPluggableService(Interface):
    pluginName = Attribute("""
    A C{str} that is the name of the plugin.
    """)
    
    isUnloadable = Attribute("""
    A C{boolean} which indicates whether the plugin can be unloaded.
    """)
