# Copyright (c) 2008-2009 Code Lounge.
# See LICENSE for details

from twisted.plugin import getPlugins, IPlugin
from igina import IPluggableService
from zope.interface import implements

import gina.plugins as plugins

class PluginMixin():
    """
    A handle Mixin for declaring Plugin
    """
    implements(IPlugin, IPluggableService)

    pluginName = 'Unknown Plugin'
    isUnloadable = False


def loadPlugins(ctrl, store=None):
    import gina.plugin as plugin
    import config
    from twisted.python import log
    
    for p in plugin.getAllPlugins():
        # XXX: Find a better way of handling this
        #if config.debug:
        log.msg('Loading Plugin: %s' % p.pluginName)
            
        if store:
            # XXX: Handle Axiom PowersUps/Downs
            plugin.installPlugin(store, p.pluginName)
            
        # Add the service to the controller
        ctrl.addHandler(p.setController(ctrl))
        
def reloadPlugins(ctrl):
    import gina.plugin as plugin
    import compileall
    
    # Drop the existing handlers    
    ctrl.dropHandlers()
    
    for p in plugin.getAllPlugins():
        exec('import %s as tmp' % p.__module__)
        reload(tmp)
        
    # Have to re-cycle through plugins to refresh twisted.plugin's cache     
    loadPlugins(ctrl, ctrl.store)
    
def getAllPlugins():
    """
    Get all plugins.
    """
    return getPlugins(IPluggableService, plugins)

def getInstalledPlugins(store):
    """
    Get all plugins installed on C{store}.
    """
    return store.powerupsFor(IPluggableService)

def getPluginProvidersByName(pluginName):
    """
    Get all objects that provide C{IPluggableService}.
    """
    for plugin in getPlugins(IPluggableService, plugins):
        if plugin.pluginName == pluginName:
            yield plugin

    raise errors.PluginNotFound(u'No plugin named "%s".' % (pluginName,))

def installPlugin(store, pluginName):
    """
    Install a plugin on a store.

    @type store: C{axiom.store.Store}

    @param pluginName: Name of the plugin to install
    @type pluginName: C{unicode}

    @raises PluginNotFound: If no plugin named C{pluginName} could be found
    """
    for plugin in getPluginProvidersByName(pluginName):
        # Quick hack until we work out why findOrCreate() isn't working
        if plugin.isUnloadable == False:
            return
        p = store.findOrCreate(plugin)
        store.powerUp(p, IPluggableService)
        return

    raise errors.PluginNotFound(u'No plugin named "%s".' % (pluginName,))

def uninstallPlugin(store, pluginName):
    """
    Uninstall a plugin from a store.

    @type store: C{axiom.store.Store}

    @param pluginName: Name of the plugin to install
    @type pluginName: C{unicode}

    @raise errors.PluginNotInstalled: If C{pluginName} is not installed on
        C{store}
    """
    for plugin in getPluginProvidersByName(pluginName):
    
        p = store.findUnique(plugin, default=None)
        if p is None:
            raise errors.PluginNotInstalled(pluginName)

        store.powerDown(p, IPluggableService)
        return
