# Copyright (c) 2008-2009 Code Lounge.
# See LICENSE for details

from twisted.python import log
from twisted.internet import reactor, utils

from gina.plugin import PluginMixin
from gina.controller import PseudoHandler

from gina.util import usage

class Feed(PseudoHandler, PluginMixin):
    help = '<b>[Feed Help]</b>'
    pluginName = 'Feed'
    
    def __init__(self, *a, **kw):
        super(Feed, self).__init__(*a, **kw)

    @usage('feed add <alias> <frequency> <url>')
    def cmd_feed_add(self, source, alias, freq, url):
        """This command adds a new feed to your list of feeds.
        
        Feeds are checked every <frequency> minutes and updates are sent to 
        you.
        
        EG: add IOL http://feeds.iol.co.za/rss/news.rss 5
        This adds a new feed for the IOL headlines which is checked every 5 
        minutes for updates.
        """
        self.protocol.msg(source, 'Feed added!')

    @usage('feed del <alias>')
    def cmd_feed_del(self, source, alias):
        """This command deletes a new feed to your list of feeds.
        """
        self.protocol.msg(source, 'Feed deleted!')

    @usage('feed list')
    def cmd_feed_list(self, source):
        """This command shows you your list of feeds.
        """
        self.protocol.msg(source, 'Feed list!')

    @usage('feed stats')
    def cmd_feed_stats(self, source):
        """This command shows some stats about feeds that Gina looks after.
        """
        self.protocol.msg(source, 'Feed stats!')

thisPlugin = Feed()
