# Copyright (c) 2008-2009 Code Lounge.
# See LICENSE for details

from twisted.python import log
from twisted.internet import reactor, utils

from gina.plugin import PluginMixin, reloadPlugins, installPlugin, uninstallPlugin
from gina.controller import PseudoHandler

from gina.util import usage

import os

class General(PseudoHandler, PluginMixin):
    help = '<b>[General Help]</b>'
    
    pluginName = 'General'
    
    def __init__(self, *a, **kw):
        super(General, self).__init__(*a, **kw)

    @usage('die <password>')
    def cmd_die(self, source, password):
        """This command shuts Gina down nicely, performing a final 
        store.transact().
        """
        self.protocol.msg(source, 'ok, bye.')
        reactor.callLater(1, reactor.stop)

    def uname_output(self, response, source):
        self.protocol.msg(source, response)

    @usage('uname')
    def cmd_uname(self, source):
        """This returns the output of `uname -a' on the system I am currently 
        running on.
        """
        paths = ['/usr/bin', '/bin']
        for path in paths:
            if os.path.exists('%s/uname' % path):
                cmd = utils.getProcessOutput('%s/uname' % path, ['-a'])
                cmd.addCallback(self.uname_output, source)

    @usage('plugins')
    def cmd_plugins(self, source):
        """This commands shows a list of all loaded plugins."""
        plugins = ', '.join([p.pluginName for p in self.ctrl.listHandlers() if
            hasattr(p, 'pluginName')])
        #for p in self.ctrl.listHandlers():
        #    if hasattr(p, 'pluginName'):
        #        plugins += '- %s\n' % p.pluginName
        self.protocol.msg(source, "<b>[Loaded Plugins]</b> %s" % (plugins,))

    @usage('reload')
    def cmd_reload(self, source):
        """This commands reloads all of Ginas plugins."""
        reloadPlugins(self.ctrl)
        self.protocol.msg(source, 'Reloaded all plugins')

    def event_new_user(self, user):
        msg = """Hi <b>%s</b>!
        I'm Gina, a jabber / XMPP bot, I can do a bunch of (hopefully) useful
        things including managing your RSS feeds and notifiying you of
        reminders. For more information about me, please see
        http://gina.codelounge.org.
        I should now appear in your buddy list and you will be able to open a
        chat window and send me commands. 
        Please type <b>help</b> for a list of what I can do.""" % (user,)
        welcome = ' '.join([l.strip() for l in msg.splitlines()])
        self.protocol.msg(user, welcome)

thisPlugin = General()
