# Copyright (c) 2008-2009 Code Lounge.
# See LICENSE for details

from twisted.python import log
from twisted.internet import reactor, utils

from gina.plugin import PluginMixin
from gina.controller import PseudoHandler

from gina.util import usage

class Reminder(PseudoHandler, PluginMixin):
    help = '<b>[Reminder Help]</b>'
    pluginName = 'Reminder'
    
    def __init__(self, *a, **kw):
        super(Reminder, self).__init__(*a, **kw)

    @usage('reminder add <time> <text>')
    def cmd_reminder_add(self, source, time, *text):
        """This command adds a reminder which Gina will let you know about at <time>

        EG: reminder add 16:30 Fetch the kids!

        <time> can be in any of the following formats:

        - 13:00     - 1 o'clock in the afternoon
        - 9:00pm    - 9 o'clock at night
        - 10h15     - Quarter past 10 in the morning
        - 5h        - In 5 hours time
        - 10m       - In 10 minutes time
        """
        msg = 'Reminder for "%s" set for %s' % (' '.join(text), time)
        self.protocol.msg(source, msg)

    @usage('reminder del <num>')
    def cmd_reminder_del(self, source, num):
        """This command removes reminder <num> from your list of set reminders.
        """
        self.protocol.msg(source, 'Reminder %s deleted.' % num)

    @usage('reminder list')
    def cmd_reminder_list(self, source):
        """Show a list of all your reminders set in Gina.
        """
        msg = '<b>Reminders</b>\n[1] 16:00 - Dont forget to blah'
        self.protocol.msg(source, msg)

thisPlugin = Reminder()
