# Eridanus
from gina import util
from gina import etree

# Mine
import dateutil.parser
from datetime import datetime

from epsilon.structlike import record

from nevow.url import URL

from twisted.python import log
from twisted.internet import reactor, utils

from gina.plugin import PluginMixin
from gina.controller import PseudoHandler
from gina.util import usage



class WundergroundConditions(record('location observationTime condition temperature humidity pressure windSpeed windDirection windChill dewPoint heatIndex')):
    @classmethod
    def fromElement(cls, node):
        location = node.findtext('display_location/full')
        #observationTime = dateutil.parser.parse(node.findtext('observation_time_rfc822'))
        observationTime = datetime.now()
        condition = node.findtext('weather')
        temp = int(node.findtext('temp_c'))
        humidity = node.findtext('relative_humidity')
        pressure = node.findtext('pressure_string')
        windSpeed = int(node.findtext('wind_mph')) * 1.609344
        windDirection = node.findtext('wind_dir')
        dewPoint = int(node.findtext('dewpoint_c'))

        heatIndex = node.findtext('heat_index_c')
        if heatIndex is None or heatIndex == 'NA':
            heatIndex = None
        else:
            heatIndex = int(heatIndex)

        windChill = node.findtext('windchill_c')
        if windChill is None or windChill == 'NA':
            windChill = None
        else:
            windChill = int(windChill)

        return cls(location=location,
                   observationTime=observationTime,
                   condition=condition,
                   temperature=temp,
                   humidity=humidity,
                   pressure=pressure,
                   windSpeed=windSpeed,
                   windDirection=windDirection,
                   windChill=windChill,
                   dewPoint=dewPoint,
                   heatIndex=heatIndex)

    @property
    def display(self):
        def temp(v):
            #return u'%d\N{DEGREE SIGN}C' % (v,)
            return u'%d&deg;C' % (v,)

        def attrs():
            if self.temperature is not None:
                yield u'Temperature', temp(self.temperature)
            if self.condition:
                yield u'Conditions', self.condition
            if self.humidity:
                yield u'Humidity', self.humidity
            if self.dewPoint:
                yield u'Dew point', temp(self.dewPoint)
            if self.pressure:
                yield u'Pressure', self.pressure
            if self.windSpeed and self.windDirection:
                yield u'Wind', u'%s at %0.2fkm/h' % (self.windDirection, self.windSpeed)
            if self.windChill:
                yield u'Wind chill', temp(self.windChill)

        timestring = unicode(self.observationTime.strftime('%H:%M %Z on %d %B %Y'))
        params = u'; '.join(u'<b>%s</b>: %s' % (key, value) for key, value in attrs())
        return u'In %s at %s: %s' % (self.location, timestring, params)


class Weather(PseudoHandler, PluginMixin):
    help = '<b>[Weather Help]</b>'
    pluginName = 'Weather'
    API_ROOT = URL.fromString('http://api.wunderground.com/auto/wui/geo')

    def __init__(self, *a, **kw):
        super(Weather, self).__init__(*a, **kw)

    def send(self, weather, source):
        weather_data = weather.display
        self.protocol.msg(source, weather_data.encode("ascii", "ignore"))

    @usage('weather current <place>')
    def cmd_weather_current(self, source, *place):
        """This command returns the current weather in <place>
        
        EG: weather current "Durban, South Africa"
        """
        url = self.API_ROOT.child('WXCurrentObXML').child('index.xml').add('query', " ".join(place))
        weather_data = util.PerseverantDownloader(url).go()
        weather_data.addCallback(lambda (data, headers): etree.fromstring(data))
        weather_data.addCallback(WundergroundConditions.fromElement)
        weather_data.addCallback(self.send, source)


thisPlugin = Weather()
