# Copyright (c) 2008-2009 Code Lounge.
# See LICENSE for details

from twisted.trial.unittest import TestCase

from gina.controller import PseudoHandler, GinaController, MethodCommand
from gina.errors import UsageError
from gina.util import usage


class TestingController(object):
    def __init__(self):
        self.store = None


class TestingHandler(PseudoHandler):
    def cmd_known(self, pseudo):
        return 201
    cmd_known.id = 200

    def cmd_varargs(self, pseudo, a, b, *c):
        return 203

    def cmd_novarargs(self, pseudo, a, b, c):
        return 204

    @usage('LIMITS foo [bar] [baz]', min=1, max=3)
    def cmd_limits(self, pseudo, a, *bc):
        return 205

    def cmd_maybebrokenname(self, pseudo):
        pass

    def event_known(self):
        return 206


class EventHandlingTests(TestCase):
    def setUp(self):
        self.ctrl = TestingController()
        self.client = TestingHandler()

    def test_locateUnknownEvent(self):
        """
        Locating an unknown event handler.
        """
        handler = self.client.locateHandler('doesnotexist')
        self.assertIdentical(handler, None)

    def test_locateKnownEvent(self):
        """
        Locating an unknown event handler.
        """
        handler = self.client.locateHandler('known')
        self.assertEquals(handler(), 206)

